'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var compositionApi = require('@vue/composition-api');
var FormulateForm = _interopDefault(require('@braid/vue-formulate/src/FormulateForm'));
var IMask = _interopDefault(require('imask'));
var FormulateInput = _interopDefault(require('@braid/vue-formulate/src/FormulateInput'));
var utils = require('@braid/vue-formulate/src/libs/utils');
var Formulate = _interopDefault(require('@braid/vue-formulate/src/Formulate'));

class Hooks {
  constructor(hooks = []) {
    this.defaultHook = (v) => v;
    return this.setHooks(hooks)
  }

  // chainables

  parse(hooks = []) {
    const _hookList = Array.isArray(hooks) ? hooks : [];
    for (let _hook of _hookList) {
      if (this.isProperHook(_hook)) {
        this.hooks.push(_hook);
      }
    }
    return this
  }

  addHook(hook) {
    if (this.isProperHook(hook) && this.isNewHook(hook)) {
      this.hooks.push(hook);
    }
    return this
  }

  setHooks(hooks = []) {
    this.hooks = hooks.filter(this.isProperHook);
    return this
  }

  setDefault(hook) {
    this.defaultHook = hook;
    return this
  }

  // not chainables

  isProperHook(hook) {
    return hook && typeof hook === 'object' && typeof hook.handler === 'function'
  }

  isNewHook(hook) {
    return this.hooks.findIndex((h) => h === hook) < 0
  }

  getHooks() {
    return this.hooks
  }

  empty() {
    return this.hooks.length === 0
  }

  apply(value, options) {
    if (this.empty()) {
      return this.defaultHook(value, options)
    } else {
      return this.hooks.reduce((result, hook) => hook.handler(result, options), value)
    }
  }

  asSingleHook() {
    const self = this;
    return {
      handler(value, options) {
        return self.apply(value, options)
      },
    }
  }
}

var hooksProp = {
  type: Object,
  default: () => ({
    model: [],
    schema: [],
    schemaOptions: [],
    schemaNode: [],
    schemaComponent: [],
  }),
};

var textMask = {
  hooks: {
    model: [
      {
        handler(value, { context }) {
          const hasMask = 'mask' in context.attributes || 'vfe-mask' in context.attributes;
          if (context.classification === 'text' && hasMask) {
            const options = context.attributes.mask || context.attributes['vfe-mask'];
            const maskOptions = typeof options === 'object' && options.mask ? options : { mask: options };
            const masked = IMask.createMask(maskOptions);
            const resolved = masked.resolve(value);
            return resolved
          } else {
            return value
          }
        },
      },
    ],
  },
};

var numberField = {
  hooks: {
    model: [
      {
        handler(value, { context }) {
          if (context.type === 'number' && typeof value === 'string') {
            return +value
          } else {
            return value
          }
        },
      },
    ],
  },
};

//
var script = {
  extends: FormulateInput,
  props: {
    modelHook: {
      type: [Function, Object, Array],
      default: null
    }
  },
  watch: {
    "context.model": {
      handler(newModel, oldModel) {
        const _modelHook = new Hooks();
        if (Array.isArray(this.modelHook)) {
          this.modelHook.map(m => _modelHook.addHook(m));
        } else if (typeof this.modelHook === "function") {
          _modelHook.addHook({ handler: this.modelHook });
        } else {
          _modelHook.addHook(this.modelHook);
        }

        if ("vfe-number" in this.context.attributes) {
          numberField.hooks.model.map(m => _modelHook.addHook(m));
        }

        if ("vfe-mask" in this.context.attributes) {
          textMask.hooks.model.map(m => _modelHook.addHook(m));
        }

        if (newModel !== oldModel) {
          this.context.model = _modelHook.apply(newModel, {
            oldModel,
            context: this.context
          });
        }
      }
    }
  }
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__ = script;

/* template */

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = undefined;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = /*#__PURE__*/normalizeComponent(
    {},
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

function leaf(item, index, { hooks, h, state } = {}) {
  if (item && typeof item === 'object' && !Array.isArray(item)) {
    const { children = null, component = __vue_component__, depth = 1, modelHook, ...attrs } = item;
    const type = component === __vue_component__ ? attrs.type || 'text' : '';
    const name = attrs.name || type || 'el';
    const key = attrs.id || `${name}-${depth}-${index}`;
    const els = Array.isArray(children) ? children.map((child) => Object.assign(child, { depth: depth + 1 })) : children;
    const _modelHook = new Hooks()
      .setHooks(hooks.model)
      .addHook(modelHook)
      .asSingleHook();

    const node = Object.assign({ type, key, depth, component, definition: { attrs: { ...attrs, modelHook: _modelHook } }, children: tree(els, { hooks, h, state }) });

    return new Hooks()
      .setHooks(hooks.schemaNode)
      .setDefault(() => node)
      .apply(node, { state })
  }
  return null
}

function tree(schema, { hooks, h, state } = {}) {
  if (Array.isArray(schema)) {
    return schema.map((el, idx) => {
      const item = leaf(el, idx, { hooks, h, state });
      return new Hooks()
        .setHooks(hooks.schemaComponent)
        .setDefault(() => h(item.component, item.definition, item.children))
        .apply(item)
    })
  }
  return schema
}

var FormulateSchema = compositionApi.defineComponent({
  name: 'FormulateSchema',
  props: {
    schema: FormulateForm.props.schema,
    hooks: hooksProp,
  },
  setup(props, { emit }) {
    const schemaHooks = new Hooks().setHooks(props.hooks.schema);
    const schemaOptionsHooks = new Hooks().setHooks(props.hooks.schemaOptions).setDefault((x) => x);
    return () => {
      return schemaHooks.apply(compositionApi.createElement('div', {}, tree(props.schema, schemaOptionsHooks.apply({ hooks: props.hooks, h: compositionApi.createElement, state: {} }, { emit, props }))), { emit, props })
    }
  },
});

//

var script$1 = {
  extends: FormulateForm,
  components: {
    FormulateSchema
  },
  props: {
    hooks: hooksProp
  },
  computed: {
    cleanedHooks() {
      const _hooks = hooksProp.default();

      Object.keys(_hooks).forEach(key => {
        _hooks[key] = new Hooks().parse(this.hooks[key]).getHooks();
      });

      return Formulate.merge(this.$formulate.options.hooks || {}, _hooks);
    }
  },
  watch: {
    formulateValue: {
      handler(values) {
        /**
         * Overrides the default behavior to properly share v-model updates
         */
        if (this.isVmodeled && values && typeof values === "object") {
          const keys = Array.from(
            new Set(Object.keys(values).concat(Object.keys(this.proxy)))
          );
          keys.forEach(field => {
            if (!utils.shallowEqualObjects(values[field], this.proxy[field])) {
              this.setFieldValue(field, values[field]);
              if (
                this.registry.has(field) &&
                !utils.shallowEqualObjects(
                  values[field],
                  this.registry.get(field).proxy
                )
              ) {
                this.registry.get(field).context.model = values[field];
              }
            }
          });
        }
      },
      deep: true
    }
  }
};

/* script */
const __vue_script__$1 = script$1;

/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "form",
    {
      class: _vm.classes,
      on: {
        submit: function($event) {
          $event.preventDefault();
          return _vm.formSubmitted($event)
        }
      }
    },
    [
      _vm.schema
        ? _c("FormulateSchema", {
            attrs: { schema: _vm.schema, hooks: _vm.cleanedHooks },
            on: {
              events: function($event) {
                return _vm.$emit("events", $event)
              }
            }
          })
        : _vm._e(),
      _vm._v(" "),
      !_vm.hasFormErrorObservers
        ? _c("FormulateErrors", { attrs: { context: _vm.formContext } })
        : _vm._e(),
      _vm._v(" "),
      _vm._t("default")
    ],
    2
  )
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__$1 = undefined;
  /* scoped */
  const __vue_scope_id__$1 = undefined;
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$1 = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    false,
    undefined,
    undefined,
    undefined
  );

class EventBus {
  constructor() {
    this.store = new Map();
  }

  on(type, handler) {
    const handlers = this.store.get(type);
    const added = handlers && handlers.push(handler);
    if (!added) {
      this.store.set(type, [handler]);
    }
  }

  off(type, handler) {
    const handlers = this.store.get(type);
    const index = handlers.indexOf(handler);
    if (handlers && index > -1) {
      handlers.splice(index, 1);
    }
  }

  emit(type, payload) {
    this.store
      .get(type)
      .slice()
      .map((handler) => handler(payload));
  }
}

var formEvents = {
  hooks: {
    schemaOptions: [
      {
        handler(options, { emit } = {}) {
          options.state = options.state || {};
          options.state.eventBus = new EventBus();
          options.state.eventBus.on('events', (payload) => emit('events', payload));
          return options
        },
      },
    ],
    schemaNode: [
      {
        handler(node, { state } = {}) {
          const { events = [], on = {}, ...attrs } = node.definition.attrs;
          node.definition.attrs = attrs;
          node.definition.on = {
            ...on,
            ...events.reduce((onEvents, eventName) => {
              onEvents[eventName] = function(element) {
                if (on[eventName]) on[eventName](element);
                state.eventBus.emit('events', { name: eventName, node: { name: node.name, type: node.type, key: node.key }, element });
              };
              return onEvents
            }, {}),
          };

          return node
        },
      },
    ],
  },
};

const components = { FormulateForm: __vue_component__$1, FormulateSchema, FormulateInput: __vue_component__ };
const features = { formEvents, textMask, numberField };

function index(options = {}) {
  let extended = {
    options: Formulate.merge(
      {
        features: {
          formEvents: true,
          textMask: false,
          numberField: false,
        },
        override: {
          FormulateForm: true,
          FormulateInput: true,
          FormulateSchema: true,
        },
      },
      options
    ),
  };

  return function plugin(instance) {
    if (extended.options.override.FormulateForm) {
      instance.extend({ components: { FormulateForm: __vue_component__$1 } });
    }

    if (extended.options.override.FormulateInput) {
      instance.extend({ components: { FormulateInput: __vue_component__ } });
    }

    if (extended.options.override.FormulateSchema) {
      instance.extend({ components: { FormulateSchema } });
    }

    if (extended.options.features.formEvents) {
      instance.extend(formEvents);
    }

    if (extended.options.features.textMask) {
      instance.extend(textMask);
    }

    if (extended.options.features.numberField) {
      instance.extend(numberField);
    }
  }
}

exports.components = components;
exports.default = index;
exports.features = features;
